#!/usr/bin/env python3

from sprun import spr

commands = []
commands.append(["echo", " "])
commands.append(["echo   "])
commands.append(["echo"])
result = spr.run(commands, spr.Proceed.ASK, spr.Silent.OK)
assert(result.commands_error == [["echo   "]])



"""
    An opinionated subprocess execution module for some special use cases

"""
import subprocess
import sys
from enum import Enum
from typing import List

Command = List[str]
CommandList = List[Command]


class Proceed(Enum):
    """ Continuation strategy for failing commands in run_commands call
    """
    CONTINUE = 1
    STOP = 2
    ASK = 3


class Silent(Enum):
    """ Continuation strategy for failing commands in run_commands call
    """
    NONE = 0
    ALL = 1
    OK = 2
    FAILURE = 3


class Result():
    """ Carrier for returning the result of the run calls
    """

    def __init__(self) -> None:
        self.commands_ok: CommandList = []
        self.commands_error: CommandList = []

    def success(self) -> bool:
        """ returns True if no command was reported returning an error
        """
        return not self.commands_error

    def failure(self) -> bool:
        """ returns True if some command was reported returning an error
        """
        return self.commands_error


def report_command(prefix: str, command: Command, *args, **kwargs) -> None:
    """ Use print to write out a command
        The command will simply be joined by space
        All other arguments will be forwarded to print
    """
    print(prefix, " ".join(command), *args, **kwargs)


def run(commands: CommandList, on_error: Proceed,
        silent: Silent = Silent.NONE):
    """ Run given commands.

        commands is a list of subprocess commands
        on_error must be a Proceed value

        If a command fails, the given Proceeds strategy is a applied

        Silent sets up the report strategy.

        Retruns a Result containing the successful and failed Command calls
    """
    silent_error = silent in [Silent.ALL, Silent.FAILURE]
    silent_ok = silent in [Silent.ALL, Silent.OK]

    def report_success(cmd: Command) -> None:
        report_command("OK:", cmd, file=sys.stdout)

    def report_error(cmd: Command) -> None:
        report_command("FAIL:", cmd, file=sys.stderr)

    def resume():
        return input("Command failed, continue with execution? (y/n): ") == "y"

    result = Result()
    for cmd in commands:
        call_rc = 1
        try:
            call_rc = subprocess.call(cmd)
        except (subprocess.SubprocessError, FileNotFoundError):
            pass
        if call_rc == 0:
            if not silent_ok:
                report_success(cmd)
            result.commands_ok.append(cmd)
        else:
            if not silent_error:
                report_error(cmd)
            result.commands_error.append(cmd)
            if on_error == Proceed.CONTINUE:
                continue
            if on_error == Proceed.STOP:
                return result
            if on_error == Proceed.ASK:
                if not resume():
                    return result

    return result



import sys

from unittest import TestCase, runner
from sprun import spr

class TestExecution(TestCase):
    def test_all_pass(self):
        commands = []
        commands.append(["echo", " "])
        commands.append(["echo"])
        result = spr.run(commands, spr.Proceed.CONTINUE)
        self.assertTrue(len(result.commands_ok) == 2)
        self.assertTrue(result.success())
        self.assertFalse(result.failure())

    def test_second_fails_and_continues(self):
        commands = []
        commands.append(["echo", " "])
        commands.append(["echo  "])
        commands.append(["echo"])
        result = spr.run(commands, spr.Proceed.CONTINUE)
        self.assertFalse(result.success())
        self.assertTrue(result.failure())
        self.assertTrue(len(result.commands_ok) == 2)
        self.assertTrue(len(result.commands_error) == 1)

    def test_second_fails_and_break(self):
        commands = []
        commands.append(["echo", " "])
        commands.append(["echo   "])
        commands.append(["echo"])
        result = spr.run(commands, spr.Proceed.STOP, spr.Silent.ALL)
        self.assertFalse(result.success())
        self.assertTrue(result.failure())
        self.assertTrue(len(result.commands_ok) == 1)
        self.assertTrue(len(result.commands_error) == 1)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sprun",
    version="0.0.3",
    author="Harald Achitz",
    author_email="harald.achitz@gmail.com",
    description="subprocess run, simply execute a list of commands",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/a4z/sprun",
    project_urls={
        "Bug Tracker": "https://gitlab.com/a4z/sprun/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)
